(function (Drupal) {

  /**
   * Alias for document.querySelectorAll.
   *
   * @param {string} s
   * @param {HTMLElement|Document} context
   * @returns {HTMLElement[]}
   */
  function $ (s, context = undefined) {
    context = context || document;
    return [...(context.querySelectorAll(s) || [])];
  }

  /**
   * Get the current index.
   *
   * @returns {number}
   */
  function getCurrentIndex() {
    let elements = $('.monarch-multistep-section');
    let currentIndex = elements.findIndex(elem => elem.classList.contains('current-step'));

    if (currentIndex >= 0) {
      return currentIndex;
    }

    let firstHiddenStep = elements.findIndex(elem => elem.classList.contains('hidden-step'));

    if (firstHiddenStep >= 0) {
      return Math.max(0, firstHiddenStep - 1);
    }

    return elements.length - 1;
  }

  /**
   * Update classes on all multi-step sections.
   *
   * @param {number} currentIndex
   */
  function updateElements(currentIndex = 0) {
    $('.monarch-multistep-section').forEach((elem, index, elements) => {
      /** @type {HTMLButtonElement|null} */
      let button = elem.querySelector('.monarch-multistep-next');

      if (button) {
        let hasRequiredInputs = $('input[required]', elem).some(input => input.value === null || input.value === undefined || input.value === '');
        button.disabled = hasRequiredInputs;
      }

      if (index <= currentIndex) {
        elem.classList.remove('hidden-step');
      }

      if (index === elements.length - 1) {
        elem.classList.add('last-step');
      }
      else {
        elem.classList.remove('last-step');
      }

      if (index === currentIndex) {
        if (!elem.classList.contains('current-step')) {
          elem.classList.add('current-step');
  
          if (!elem.open) {
            elem.open = true;
            elem.querySelector('summary').setAttribute('aria-expanded', 'true');
          }
        }
      }
      else {
        if (elem.classList.contains('current-step')) {
          elem.classList.remove('current-step');
  
          if (elem.open) {
            elem.open = false;
            elem.querySelector('summary').setAttribute('aria-expanded', 'false');
          }
        }
      }
    });
  }

  /**
   * Respond to a click event.
   *
   * @param {PointerEvent} e
   */
  function clickNext (e) {
    updateElements(getCurrentIndex() + 1);
  }

  /**
   * Respond to input events.
   */
  function inputChange () {
    updateElements(getCurrentIndex());
  }

  function updateAll() {
    $('.monarch-multistep-next').forEach(button => {
      button.removeEventListener('click', clickNext);
      button.addEventListener('click', clickNext);
    });

    $('.monarch-multistep-section input').forEach(input => { 
      ['change', 'keyup', 'blur'].forEach(eventName => {
        input.removeEventListener(eventName, inputChange);
        input.addEventListener(eventName, inputChange);
      });
    });

    updateElements(getCurrentIndex());
  }

  Drupal.behaviors.monarch_multistep_field_group = {
    attach() {
      updateAll();
    },
  };
})(Drupal);
